@@ -49,9 +49,7 @@
  * In short: designed for maximum evilness.
  */
 
-#include "gem_stress.h"
-
-#include <signal.h>
+#include "rendercopy.h"
 
 #define CMD_POLY_STIPPLE_OFFSET       0x7906
 
@@ -67,15 +65,6 @@
  *   first one (to check consistency of the kernel recovery paths)
  */
 
-static uint64_t gem_aperture_size(int fd)
-{
-	struct drm_i915_gem_get_aperture aperture;
-
-	aperture.aper_size = 256*1024*1024;
-	(void)drmIoctl(fd, DRM_IOCTL_I915_GEM_GET_APERTURE, &aperture);
-	return aperture.aper_size;
-}
-
 drm_intel_bufmgr *bufmgr;
 struct intel_batchbuffer *batch;
 int drm_fd;
@@ -84,8 +73,33 @@ int num_fences;
 
 drm_intel_bo *busy_bo;
 
+struct option_struct {
+    unsigned scratch_buf_size;
+    unsigned max_dimension;
+    unsigned num_buffers;
+    int trace_tile;
+    int no_hw;
+    int gpu_busy_load;
+    int use_render;
+    int use_blt;
+    int forced_tiling;
+    int use_cpu_maps;
+    int total_rounds;
+    int fail;
+    int tiles_per_buf;
+    int ducttape;
+    int tile_size;
+    int check_render_cpyfn;
+    int use_signal_helper;
+};
+
 struct option_struct options;
 
+#define MAX_BUFS		4096
+#define SCRATCH_BUF_SIZE	1024*1024
+#define BUSY_BUF_SIZE		(256*4096)
+#define TILE_BYTES(size)	((size)*(size)*sizeof(uint32_t))
+
 static struct scratch_buf buffers[2][MAX_BUFS];
 /* tile i is at logical position tile_permutation[i] */
 static unsigned *tile_permutation;
@@ -102,29 +116,6 @@ struct {
 	unsigned max_failed_reads;
 } stats;
 
-static void signal_helper_process(pid_t pid)
-{
-	/* Interrupt the parent process at 500Hz, just to be annoying */
-	while (1) {
-		usleep(1000 * 1000 / 500);
-		if (kill(pid, SIGUSR1)) /* Parent has died, so must we. */
-			exit(0);
-	}
-}
-
-static pid_t fork_signal_helper(void)
-{
-	pid_t pid;
-
-	pid = fork();
-	if (pid == 0) {
-		signal_helper_process(getppid());
-		return -1;
-	}
-
-	return pid;
-}
-
 static void tile2xy(struct scratch_buf *buf, unsigned tile, unsigned *x, unsigned *y)
 {
 	assert(tile < buf->num_tiles);
@@ -177,7 +168,7 @@ static void emit_blt(drm_intel_bo *src_bo, uint32_t src_tiling, unsigned src_pit
 
 /* All this gem trashing wastes too much cpu time, so give the gpu something to
  * do to increase changes for races. */
-void keep_gpu_busy(void)
+static void keep_gpu_busy(void)
 {
 	int tmp;
 
@@ -188,22 +179,10 @@ void keep_gpu_busy(void)
 		 busy_bo, 0, 4096, 0, 128);
 }
 
-static void set_to_cpu_domain(struct scratch_buf *buf, int write)
+static void set_to_cpu_domain(struct scratch_buf *buf, int writing)
 {
-	struct drm_i915_gem_set_domain set_domain;
-	int ret;
-
-	set_domain.handle = buf->bo->handle;
-	set_domain.read_domains = I915_GEM_DOMAIN_CPU;
-	if (write)
-		set_domain.write_domain = I915_GEM_DOMAIN_CPU;
-	else
-		set_domain.write_domain = 0;
-
-	ret = drmIoctl(drm_fd,
-		       DRM_IOCTL_I915_GEM_SET_DOMAIN,
-		       &set_domain);
-	assert(ret == 0);
+	gem_set_domain(drm_fd, buf->bo->handle, I915_GEM_DOMAIN_CPU,
+		       writing ? I915_GEM_DOMAIN_CPU : 0);
 }
 
 static unsigned int copyfunc_seq = 0;
@@ -343,22 +322,36 @@ static void render_copyfunc(struct scratch_buf *src, unsigned src_x, unsigned sr
 			    struct scratch_buf *dst, unsigned dst_x, unsigned dst_y,
 			    unsigned logical_tile_no)
 {
+	static unsigned keep_gpu_busy_counter = 0;
+
+	/* check both edges of the fence usage */
+	if (keep_gpu_busy_counter & 1)
+		keep_gpu_busy();
+
 	if (IS_GEN2(devid))
-		gen2_render_copyfunc(src, src_x, src_y,
-				     dst, dst_x, dst_y,
-				     logical_tile_no);
+		gen2_render_copyfunc(batch,
+				     src, src_x, src_y,
+				     options.tile_size, options.tile_size,
+				     dst, dst_x, dst_y);
 	else if (IS_GEN3(devid))
-		gen3_render_copyfunc(src, src_x, src_y,
-				     dst, dst_x, dst_y,
-				     logical_tile_no);
+		gen3_render_copyfunc(batch,
+				     src, src_x, src_y,
+				     options.tile_size, options.tile_size,
+				     dst, dst_x, dst_y);
 	else if (IS_GEN6(devid))
-		gen6_render_copyfunc(src, src_x, src_y,
-				     dst, dst_x, dst_y,
-				     logical_tile_no);
+		gen6_render_copyfunc(batch,
+				     src, src_x, src_y,
+				     options.tile_size, options.tile_size,
+				     dst, dst_x, dst_y);
 	else
 		blitter_copyfunc(src, src_x, src_y,
 				 dst, dst_x, dst_y,
 				 logical_tile_no);
+	if (!(keep_gpu_busy_counter & 1))
+		keep_gpu_busy();
+
+	keep_gpu_busy_counter++;
+	intel_batchbuffer_flush(batch);
 }
 
 static void next_copyfunc(int tile)
@@ -459,10 +452,10 @@ static void sanitize_stride(struct scratch_buf *buf)
 {
 
 	if (buf_height(buf) > options.max_dimension)
-		buf->stride = options.scratch_buf_size / options.max_dimension;
+		buf->stride = buf->size / options.max_dimension;
 
 	if (buf_height(buf) < options.tile_size)
-		buf->stride = options.scratch_buf_size / options.tile_size;
+		buf->stride = buf->size / options.tile_size;
 
 	if (buf_width(buf) < options.tile_size)
 		buf->stride = options.tile_size * sizeof(uint32_t);
@@ -479,6 +472,7 @@ static void sanitize_stride(struct scratch_buf *buf)
 static void init_buffer(struct scratch_buf *buf, unsigned size)
 {
 	buf->bo = drm_intel_bo_alloc(bufmgr, "tiled bo", size, 4096);
+	buf->size = size;
 	assert(buf->bo);
 	buf->tiling = I915_TILING_NONE;
 	buf->stride = 4096;
@@ -498,19 +492,6 @@ static void init_buffer(struct scratch_buf *buf, unsigned size)
 	buf->num_tiles = options.tiles_per_buf;
 }
 
-static void permute_array(void *array, unsigned size,
-			  void (*exchange_func)(void *array, unsigned i, unsigned j))
-{
-	int i;
-
-	for (i = size - 1; i > 1; i--) {
-		/* yes, not perfectly uniform, who cares */
-		long l = random() % (i +1);
-		if (i != l)
-			exchange_func(array, i, l);
-	}
-}
-
 static void exchange_buf(void *array, unsigned i, unsigned j)
 {
 	struct scratch_buf *buf_arr, tmp;
@@ -522,38 +503,12 @@ static void exchange_buf(void *array, unsigned i, unsigned j)
 }
 
 
-/* libdrm is too clever and prevents us from changing tiling of buffers already
- * used in relocations. */
-static void set_tiling(drm_intel_bo *bo, unsigned *tiling, unsigned stride)
-{
-	struct drm_i915_gem_set_tiling set_tiling;
-	int ret;
-
-	memset(&set_tiling, 0, sizeof(set_tiling));
-	do {
-		/* set_tiling is slightly broken and overwrites the
-		 * input on the error path, so we have to open code
-		 * drmIoctl.
-		 */
-		set_tiling.handle = bo->handle;
-		set_tiling.tiling_mode = *tiling;
-		set_tiling.stride = tiling ? stride : 0;
-
-		ret = ioctl(drm_fd,
-			    DRM_IOCTL_I915_GEM_SET_TILING,
-			    &set_tiling);
-	} while (ret == -1 && (errno == EINTR || errno == EAGAIN));
-	assert(ret != -1);
-
-	*tiling = set_tiling.tiling_mode;
-}
-
 static void init_set(unsigned set)
 {
 	long int r;
 	int i;
 
-	permute_array(buffers[set], num_buffers, exchange_buf);
+	drmtest_permute_array(buffers[set], num_buffers, exchange_buf);
 
 	if (current_set == 1 && options.gpu_busy_load == 0) {
 		gpu_busy_load++;
@@ -591,9 +546,9 @@ static void init_set(unsigned set)
 
 		sanitize_stride(&buffers[set][i]);
 
-		set_tiling(buffers[set][i].bo,
-			   &buffers[set][i].tiling,
-			   buffers[set][i].stride);
+		gem_set_tiling(drm_fd, buffers[set][i].bo->handle,
+			       buffers[set][i].tiling,
+			       buffers[set][i].stride);
 
 		if (options.trace_tile != -1 && i == options.trace_tile/options.tiles_per_buf)
 			printf("changing buffer %i containing tile %i: tiling %i, stride %i\n", i,
@@ -857,11 +812,8 @@ static void init(void)
 	batch = intel_batchbuffer_alloc(bufmgr, devid);
 
 	busy_bo = drm_intel_bo_alloc(bufmgr, "tiled bo", BUSY_BUF_SIZE, 4096);
-	if (options.forced_tiling >= 0) {
-		tmp = options.forced_tiling;
-		set_tiling(busy_bo, &tmp, 4096);
-		assert(tmp == options.forced_tiling);
-	}
+	if (options.forced_tiling >= 0)
+		gem_set_tiling(drm_fd, busy_bo->handle, options.forced_tiling, 4096);
 
 	for (i = 0; i < num_buffers; i++) {
 		init_buffer(&buffers[0][i], options.scratch_buf_size);
@@ -924,7 +876,6 @@ int main(int argc, char **argv)
 {
 	int i, j;
 	unsigned *current_permutation, *tmp_permutation;
-	pid_t signal_helper = -1;
 
 	drm_fd = drm_open_any();
 	devid = intel_get_drm_devid(drm_fd);
@@ -932,9 +883,8 @@ int main(int argc, char **argv)
 	parse_options(argc, argv);
 
 	/* start our little helper early before too may allocations occur */
-	signal(SIGUSR1, SIG_IGN);
 	if (options.use_signal_helper)
-		signal_helper = fork_signal_helper();
+		drmtest_fork_signal_helper();
 
 	init();
 
@@ -961,7 +911,7 @@ int main(int argc, char **argv)
 
 		for (j = 0; j < num_total_tiles; j++)
 			current_permutation[j] = j;
-		permute_array(current_permutation, num_total_tiles, exchange_uint);
+		drmtest_permute_array(current_permutation, num_total_tiles, exchange_uint);
 
 		copy_tiles(current_permutation);
 
@@ -984,8 +934,7 @@ int main(int argc, char **argv)
 
 	close(drm_fd);
 
-	if (signal_helper != -1)
-		kill(signal_helper, SIGQUIT);
+	drmtest_stop_signal_helper();
 
 	return 0;
 }
