@@ -65,52 +65,6 @@ static inline uint32_t pack_float(float f)
 	return u.dw;
 }
 
-static uint32_t gem_create(int fd, int size)
-{
-	struct drm_i915_gem_create create;
-
-	create.handle = 0;
-	create.size = size;
-	(void)drmIoctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create);
-	assert(create.handle);
-
-	return create.handle;
-}
-
-static void gem_close(int fd, uint32_t handle)
-{
-	struct drm_gem_close close;
-	int ret;
-
-	close.handle = handle;
-	ret = drmIoctl(fd, DRM_IOCTL_GEM_CLOSE, &close);
-	assert(ret == 0);
-}
-
-static uint64_t
-gem_aperture_size(int fd)
-{
-	struct drm_i915_gem_get_aperture aperture;
-
-	aperture.aper_size = 512*1024*1024;
-	(void)drmIoctl(fd, DRM_IOCTL_I915_GEM_GET_APERTURE, &aperture);
-	return aperture.aper_size;
-}
-
-static void
-gem_write(int fd, uint32_t handle, int offset, int size, const void *buf)
-{
-	struct drm_i915_gem_pwrite pwrite;
-	int ret;
-
-	pwrite.handle = handle;
-	pwrite.offset = offset;
-	pwrite.size = size;
-	pwrite.data_ptr = (uintptr_t)buf;
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_PWRITE, &pwrite);
-	assert(ret == 0);
-}
-
 static uint32_t fill_reloc(struct drm_i915_gem_relocation_entry *reloc,
 			   uint32_t offset,
 			   uint32_t handle,
@@ -316,7 +270,7 @@ render_copy(int fd,
 
 	assert(b - batch <= 1024);
 	handle = gem_create(fd, 4096);
-	gem_write(fd, handle, 0, (b-batch)*sizeof(batch[0]), batch);
+	gem_write(fd, handle, 0, batch, (b-batch)*sizeof(batch[0]));
 
 	assert(r-reloc == 2);
 
@@ -397,7 +351,7 @@ static void blt_copy(int fd, uint32_t dst, uint32_t src)
 
 	assert(b - batch <= 1024);
 	handle = gem_create(fd, 4096);
-	gem_write(fd, handle, 0, (b-batch)*sizeof(batch[0]), batch);
+	gem_write(fd, handle, 0, batch, (b-batch)*sizeof(batch[0]));
 
 	assert(r-reloc == 2);
 
@@ -464,42 +418,6 @@ retry:
 	}
 }
 
-static void *gem_mmap(int fd, uint32_t handle, int size, int prot)
-{
-	struct drm_i915_gem_mmap_gtt mmap_arg;
-	void *ptr;
-
-	mmap_arg.handle = handle;
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg)) {
-		assert(0);
-		return NULL;
-	}
-
-	ptr = mmap(0, size, prot, MAP_SHARED, fd, mmap_arg.offset);
-	if (ptr == MAP_FAILED) {
-		assert(0);
-		ptr = NULL;
-	}
-
-	return ptr;
-}
-
-static void gem_set_tiling(int fd, uint32_t handle, int tiling, int stride)
-{
-	struct drm_i915_gem_set_tiling set_tiling;
-	int ret;
-
-	do {
-		set_tiling.handle = handle;
-		set_tiling.tiling_mode = tiling;
-		set_tiling.stride = stride;
-
-		ret = ioctl(fd, DRM_IOCTL_I915_GEM_SET_TILING, &set_tiling);
-	} while (ret == -1 && (errno == EINTR || errno == EAGAIN));
-	assert(ret == 0);
-	assert(set_tiling.tiling_mode == tiling);
-}
-
 static uint32_t
 create_bo(int fd, uint32_t val, int tiling)
 {
@@ -512,6 +430,7 @@ create_bo(int fd, uint32_t val, int tiling)
 
 	/* Fill the BO with dwords starting at val */
 	v = gem_mmap(fd, handle, WIDTH*HEIGHT*4, PROT_READ | PROT_WRITE);
+	assert(v);
 	for (i = 0; i < WIDTH*HEIGHT; i++)
 		v[i] = val++;
 	munmap(v, WIDTH*HEIGHT*4);
@@ -526,6 +445,7 @@ check_bo(int fd, uint32_t handle, uint32_t val)
 	int i;
 
 	v = gem_mmap(fd, handle, WIDTH*HEIGHT*4, PROT_READ);
+	assert(v);
 	for (i = 0; i < WIDTH*HEIGHT; i++) {
 		if (v[i] != val) {
 			fprintf(stderr, "Expected 0x%08x, found 0x%08x "
