@@ -50,13 +50,8 @@
 #include <assert.h>
 #include <cairo.h>
 #include <errno.h>
-#include <glib.h>
-#include <libudev.h>
 #include <math.h>
 #include <stdint.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <unistd.h>
 #include <sys/poll.h>
 #include <sys/time.h>
@@ -66,29 +61,22 @@
 #include "xf86drm.h"
 #include "xf86drmMode.h"
 #include "i915_drm.h"
+#include "drmtest.h"
+#include "testdisplay.h"
 
 #if defined(DRM_IOCTL_MODE_ADDFB2) && defined(DRM_I915_SET_SPRITE_COLORKEY)
 #define TEST_PLANES 1
 #include "drm_fourcc.h"
 #endif
 
-struct udev_monitor *uevent_monitor;
 drmModeRes *resources;
-int fd, modes;
+int drm_fd, modes;
 int dump_info = 0, test_all_modes =0, test_preferred_mode = 0, force_mode = 0,
 	test_plane, enable_tiling;
 int sleep_between_modes = 5;
 uint32_t depth = 24, stride, bpp;
 
-float force_clock;
-int	force_hdisplay;
-int	force_hsync_start;
-int	force_hsync_end;
-int	force_htotal;
-int	force_vdisplay;
-int	force_vsync_start;
-int	force_vsync_end;
-int	force_vtotal;
+drmModeModeInfo force_timing;
 
 int crtc_x, crtc_y, crtc_w, crtc_h, width, height;
 unsigned int plane_fb_id;
@@ -102,11 +90,11 @@ uint32_t *fb_ptr;
 
 struct type_name {
 	int type;
-	char *name;
+	const char *name;
 };
 
 #define type_name_fn(res) \
-static char * res##_str(int type) {			\
+static const char * res##_str(int type) {			\
 	unsigned int i;					\
 	for (i = 0; i < ARRAY_SIZE(res##_names); i++) { \
 		if (res##_names[i].type == type)	\
@@ -188,19 +176,28 @@ static void dump_mode(drmModeModeInfo *mode)
 	       mode->clock);
 }
 
-static void dump_connectors(void)
+
+static void dump_connectors_fd(int drmfd)
 {
 	int i, j;
 
+	drmModeRes *mode_resources = drmModeGetResources(drmfd);
+
+	if (!mode_resources) {
+		fprintf(stderr, "drmModeGetResources failed: %s\n",
+			strerror(errno));
+		return;
+	}
+
 	printf("Connectors:\n");
 	printf("id\tencoder\tstatus\t\ttype\tsize (mm)\tmodes\n");
-	for (i = 0; i < resources->count_connectors; i++) {
+	for (i = 0; i < mode_resources->count_connectors; i++) {
 		drmModeConnector *connector;
 
-		connector = drmModeGetConnector(fd, resources->connectors[i]);
+		connector = drmModeGetConnector(drmfd, mode_resources->connectors[i]);
 		if (!connector) {
 			fprintf(stderr, "could not get connector %i: %s\n",
-				resources->connectors[i], strerror(errno));
+				mode_resources->connectors[i], strerror(errno));
 			continue;
 		}
 
@@ -224,21 +221,24 @@ static void dump_connectors(void)
 		drmModeFreeConnector(connector);
 	}
 	printf("\n");
+
+	drmModeFreeResources(mode_resources);
 }
 
-static void dump_crtcs(void)
+static void dump_crtcs_fd(int drmfd)
 {
 	int i;
+	drmModeRes *mode_resources = drmModeGetResources(drmfd);
 
 	printf("CRTCs:\n");
 	printf("id\tfb\tpos\tsize\n");
-	for (i = 0; i < resources->count_crtcs; i++) {
+	for (i = 0; i < mode_resources->count_crtcs; i++) {
 		drmModeCrtc *crtc;
 
-		crtc = drmModeGetCrtc(fd, resources->crtcs[i]);
+		crtc = drmModeGetCrtc(drmfd, mode_resources->crtcs[i]);
 		if (!crtc) {
 			fprintf(stderr, "could not get crtc %i: %s\n",
-				resources->crtcs[i], strerror(errno));
+				mode_resources->crtcs[i], strerror(errno));
 			continue;
 		}
 		printf("%d\t%d\t(%d,%d)\t(%dx%d)\n",
@@ -251,8 +251,11 @@ static void dump_crtcs(void)
 		drmModeFreeCrtc(crtc);
 	}
 	printf("\n");
+
+	drmModeFreeResources(mode_resources);
 }
 
+
 #ifdef TEST_PLANES
 static void dump_planes(void)
 {
@@ -260,9 +263,9 @@ static void dump_planes(void)
 	drmModePlane *ovr;
 	int i;
 
-	plane_resources = drmModeGetPlaneResources(fd);
+	plane_resources = drmModeGetPlaneResources(drm_fd);
 	if (!plane_resources) {
-		fprintf(stderr, "drmModeGetPlaneResources failed: %s\n",
+		fprintf(stderr, "drmModeGetPlaneResources dump failed: %s\n",
 			strerror(errno));
 		return;
 	}
@@ -270,7 +273,7 @@ static void dump_planes(void)
 	printf("Planes:\n");
 	printf("id\tcrtc\tfb\tCRTC x,y\tx,y\tgamma size\n");
 	for (i = 0; i < plane_resources->count_planes; i++) {
-		ovr = drmModeGetPlane(fd, plane_resources->planes[i]);
+		ovr = drmModeGetPlane(drm_fd, plane_resources->planes[i]);
 		if (!ovr) {
 			fprintf(stderr, "drmModeGetPlane failed: %s\n",
 				strerror(errno));
@@ -300,7 +303,7 @@ static void connector_find_preferred_mode(struct connector *c)
 
 	/* First, find the connector & mode */
 	c->mode_valid = 0;
-	connector = drmModeGetConnector(fd, c->id);
+	connector = drmModeGetConnector(drm_fd, c->id);
 	if (!connector) {
 		fprintf(stderr, "could not get connector %d: %s\n",
 			c->id, strerror(errno));
@@ -349,7 +352,7 @@ static void connector_find_preferred_mode(struct connector *c)
 
 	/* Now get the encoder */
 	for (i = 0; i < connector->count_encoders; i++) {
-		encoder = drmModeGetEncoder(fd, connector->encoders[i]);
+		encoder = drmModeGetEncoder(drm_fd, connector->encoders[i]);
 
 		if (!encoder) {
 			fprintf(stderr, "could not get encoder %i: %s\n",
@@ -377,47 +380,12 @@ static void connector_find_preferred_mode(struct connector *c)
 	c->crtc = resources->crtcs[i];
 	c->pipe = i;
 
-	if(test_preferred_mode)
+	if(test_preferred_mode || force_mode)
 		resources->crtcs[i] = 0;
 
 	c->connector = connector;
 }
 
-static uint32_t gem_create(int fd, int size)
-{
-	struct drm_i915_gem_create create;
-
-	create.handle = 0;
-	create.size = (size + 4095) & -4096;
-	(void)drmIoctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create);
-
-	return create.handle;
-}
-
-static void *gem_mmap(int fd, uint32_t handle, int size, int prot)
-{
-	struct drm_i915_gem_mmap_gtt mmap_arg;
-	void *ptr;
-
-	mmap_arg.handle = handle;
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg))
-		return NULL;
-
-	ptr = mmap(0, size, prot, MAP_SHARED, fd, mmap_arg.offset);
-	if (ptr == MAP_FAILED)
-		ptr = NULL;
-
-	return ptr;
-}
-
-static void gem_close(int fd, uint32_t handle)
-{
-	struct drm_gem_close close;
-
-	close.handle = handle;
-	(void)drmIoctl(fd, DRM_IOCTL_GEM_CLOSE, &close);
-}
-
 static cairo_surface_t *
 allocate_surface(int fd, int width, int height, uint32_t depth,
 		 uint32_t *handle, int tiled)
@@ -699,7 +667,7 @@ connector_find_plane(struct connector *c)
 	uint32_t id = 0;
 	int i;
 
-	plane_resources = drmModeGetPlaneResources(fd);
+	plane_resources = drmModeGetPlaneResources(drm_fd);
 	if (!plane_resources) {
 		fprintf(stderr, "drmModeGetPlaneResources failed: %s\n",
 			strerror(errno));
@@ -707,7 +675,7 @@ connector_find_plane(struct connector *c)
 	}
 
 	for (i = 0; i < plane_resources->count_planes; i++) {
-		ovr = drmModeGetPlane(fd, plane_resources->planes[i]);
+		ovr = drmModeGetPlane(drm_fd, plane_resources->planes[i]);
 		if (!ovr) {
 			fprintf(stderr, "drmModeGetPlane failed: %s\n",
 				strerror(errno));
@@ -757,7 +725,7 @@ enable_plane(struct connector *c)
 	uint32_t handle;
 	int ret;
 	uint32_t handles[4], pitches[4], offsets[4]; /* we only use [0] */
-	struct drm_intel_set_sprite_destkey set;
+	struct drm_intel_sprite_colorkey set;
 	uint32_t plane_flags = 0;
 
 	plane_id = connector_find_plane(c);
@@ -767,7 +735,7 @@ enable_plane(struct connector *c)
 	}
 	plane_crtc_id = c->crtc;
 
-	surface = allocate_surface(fd, plane_width, plane_height, 24, &handle, 1);
+	surface = allocate_surface(drm_fd, plane_width, plane_height, 24, &handle, 1);
 	if (!surface) {
 		fprintf(stderr, "allocation failed %dx%d\n", plane_width, plane_height);
 		return;
@@ -786,11 +754,11 @@ enable_plane(struct connector *c)
 	pitches[0] = cairo_image_surface_get_stride(surface);
 	memset(offsets, 0, sizeof(offsets));
 	handles[0] = handles[1] = handles[2] = handles[3] = handle;
-	ret = drmModeAddFB2(fd, plane_width, plane_height, DRM_FORMAT_XRGB8888,
+	ret = drmModeAddFB2(drm_fd, plane_width, plane_height, DRM_FORMAT_XRGB8888,
 			    handles, pitches, offsets, &plane_fb_id,
 			    plane_flags);
 	cairo_surface_destroy(surface);
-	gem_close(fd, handle);
+	gem_close(drm_fd, handle);
 
 	if (ret) {
 		fprintf(stderr, "failed to add fb (%dx%d): %s\n",
@@ -799,11 +767,13 @@ enable_plane(struct connector *c)
 	}
 
 	set.plane_id = plane_id;
-	set.value = SPRITE_COLOR_KEY;
-	ret = drmCommandWrite(fd, DRM_I915_SET_SPRITE_DESTKEY, &set,
+	set.max_value = SPRITE_COLOR_KEY;
+	set.min_value = SPRITE_COLOR_KEY;
+	set.channel_mask = 0xffffff;
+	ret = drmCommandWrite(drm_fd, DRM_I915_SET_SPRITE_COLORKEY, &set,
 			      sizeof(set));
 
-	if (drmModeSetPlane(fd, plane_id, plane_crtc_id, plane_fb_id,
+	if (drmModeSetPlane(drm_fd, plane_id, plane_crtc_id, plane_fb_id,
 			    plane_flags, crtc_x, crtc_y, crtc_w, crtc_h,
 			    0, 0, plane_width, plane_height)) {
 		fprintf(stderr, "failed to enable plane: %s\n",
@@ -896,18 +866,10 @@ set_mode(struct connector *c)
 
 	test_mode_num = 1;
 	if (force_mode){
-		c->mode.clock = force_clock*1000;
-		c->mode.hdisplay = force_hdisplay;
-		c->mode.hsync_start = force_hsync_start;
-		c->mode.hsync_end = force_hsync_end;
-		c->mode.htotal = force_htotal;
-		c->mode.vdisplay = force_vdisplay;
-		c->mode.vsync_start = force_vsync_start;
-		c->mode.vsync_end = force_vsync_end;
-		c->mode.vtotal = force_vtotal;
-		c->mode.vrefresh =(force_clock*1e6)/(force_htotal*force_vtotal);
+		memcpy( &c->mode, &force_timing, sizeof(force_timing));
+		c->mode.vrefresh =(force_timing.clock*1e3)/(force_timing.htotal*force_timing.vtotal);
 		c->mode_valid = 1;
-		sprintf(c->mode.name, "%dx%d", force_hdisplay, force_vdisplay);
+		sprintf(c->mode.name, "%dx%d", force_timing.hdisplay, force_timing.vdisplay);
 	} else if (test_all_modes)
 		test_mode_num = c->connector->count_modes;
 
@@ -926,7 +888,7 @@ set_mode(struct connector *c)
 		width = c->mode.hdisplay;
 		height = c->mode.vdisplay;
 
-		surface = allocate_surface(fd, width, height, depth,
+		surface = allocate_surface(drm_fd, width, height, depth,
 					   &handle, enable_tiling);
 		if (!surface) {
 			fprintf(stderr, "allocation failed %dx%d\n", width, height);
@@ -961,11 +923,11 @@ set_mode(struct connector *c)
 			fprintf(stderr, "failed to draw pretty picture %dx%d: %s\n",
 				width, height, cairo_status_to_string(status));
 
-		ret = drmModeAddFB(fd, width, height, depth, bpp,
+		ret = drmModeAddFB(drm_fd, width, height, depth, bpp,
 				   cairo_image_surface_get_stride(surface),
 				   handle, &fb_id);
 		cairo_surface_destroy(surface);
-		gem_close(fd, handle);
+		gem_close(drm_fd, handle);
 
 		if (ret) {
 			fprintf(stderr, "failed to add fb (%dx%d): %s\n",
@@ -975,7 +937,7 @@ set_mode(struct connector *c)
 
 		fprintf(stdout, "CRTS(%u):",c->crtc);
 		dump_mode(&c->mode);
-		if (drmModeSetCrtc(fd, c->crtc, fb_id, 0, 0,
+		if (drmModeSetCrtc(drm_fd, c->crtc, fb_id, 0, 0,
 				   &c->id, 1, &c->mode)) {
 			fprintf(stderr, "failed to set mode (%dx%d@%dHz): %s\n",
 				width, height, c->mode.vrefresh,
@@ -991,9 +953,9 @@ set_mode(struct connector *c)
 
 	}
 
-	if(!test_preferred_mode){
-		drmModeRmFB(fd,fb_id);
-		drmModeSetCrtc(fd, c->crtc, fb_id, 0, 0,  &c->id, 1, 0);
+	if(test_all_modes){
+		drmModeRmFB(drm_fd,fb_id);
+		drmModeSetCrtc(drm_fd, c->crtc, fb_id, 0, 0,  &c->id, 1, 0);
 	}
 
 	drmModeFreeEncoder(c->encoder);
@@ -1009,12 +971,12 @@ set_mode(struct connector *c)
  * Each connector has a corresponding encoder, except in the SDVO case
  * where an encoder may have multiple connectors.
  */
-static int update_display(void)
+int update_display(void)
 {
 	struct connector *connectors;
 	int c;
 
-	resources = drmModeGetResources(fd);
+	resources = drmModeGetResources(drm_fd);
 	if (!resources) {
 		fprintf(stderr, "drmModeGetResources failed: %s\n",
 			strerror(errno));
@@ -1027,8 +989,8 @@ static int update_display(void)
 		return 0;
 
 	if (dump_info) {
-		dump_connectors();
-		dump_crtcs();
+		dump_connectors_fd(drm_fd);
+		dump_crtcs_fd(drm_fd);
 		dump_planes();
 	}
 
@@ -1043,8 +1005,6 @@ static int update_display(void)
 	return 1;
 }
 
-extern char *optarg;
-extern int optind, opterr, optopt;
 static char optstr[] = "hiaf:s:d:p:mt";
 
 static void usage(char *name)
@@ -1066,32 +1026,6 @@ static void usage(char *name)
 
 #define dump_resource(res) if (res) dump_##res()
 
-static gboolean hotplug_event(GIOChannel *source, GIOCondition condition,
-			      gpointer data)
-{
-	struct udev_device *dev;
-	dev_t udev_devnum;
-	struct stat s;
-	const char *hotplug;
-
-	dev = udev_monitor_receive_device(uevent_monitor);
-	if (!dev)
-		goto out;
-
-	udev_devnum = udev_device_get_devnum(dev);
-	fstat(fd, &s);
-
-	hotplug = udev_device_get_property_value(dev, "HOTPLUG");
-
-	if (memcmp(&s.st_rdev, &udev_devnum, sizeof(dev_t)) == 0 &&
-	    hotplug && atoi(hotplug) == 1)
-		update_display();
-
-	udev_device_unref(dev);
-out:
-	return TRUE;
-}
-
 static gboolean input_event(GIOChannel *source, GIOCondition condition,
 			    gpointer data)
 {
@@ -1100,24 +1034,24 @@ static gboolean input_event(GIOChannel *source, GIOCondition condition,
 
 	count = read(g_io_channel_unix_get_fd(source), buf, sizeof(buf));
 	if (buf[0] == 'q' && (count == 1 || buf[1] == '\n')) {
-		disable_planes(fd);
+		disable_planes(drm_fd);
 		exit(0);
 	} else if (buf[0] == 'a')
-		adjust_plane(fd, -10, 0, 0, 0);
+		adjust_plane(drm_fd, -10, 0, 0, 0);
 	else if (buf[0] == 'd')
-		adjust_plane(fd, 10, 0, 0, 0);
+		adjust_plane(drm_fd, 10, 0, 0, 0);
 	else if (buf[0] == 'w')
-		adjust_plane(fd, 0, -10, 0, 0);
+		adjust_plane(drm_fd, 0, -10, 0, 0);
 	else if (buf[0] == 's')
-		adjust_plane(fd, 0, 10, 0, 0);
+		adjust_plane(drm_fd, 0, 10, 0, 0);
 	else if (buf[0] == 'j')
-		adjust_plane(fd, 0, 0, 10, 0);
+		adjust_plane(drm_fd, 0, 0, 10, 0);
 	else if (buf[0] == 'l')
-		adjust_plane(fd, 0, 0, -10, 0);
+		adjust_plane(drm_fd, 0, 0, -10, 0);
 	else if (buf[0] == 'k')
-		adjust_plane(fd, 0, 0, 0, -10);
+		adjust_plane(drm_fd, 0, 0, 0, -10);
 	else if (buf[0] == 'i')
-		adjust_plane(fd, 0, 0, 0, 10);
+		adjust_plane(drm_fd, 0, 0, 0, 10);
 
 	return TRUE;
 }
@@ -1125,12 +1059,12 @@ static gboolean input_event(GIOChannel *source, GIOCondition condition,
 int main(int argc, char **argv)
 {
 	int c;
-	char *modules[] = { "i915" };
+	const char *modules[] = { "i915" };
 	unsigned int i;
-	struct udev *u;
 	int ret = 0;
-	GIOChannel *udevchannel, *stdinchannel;
+	GIOChannel *stdinchannel;
 	GMainLoop *mainloop;
+	float force_clock;
 
 	opterr = 0;
 	while ((c = getopt(argc, argv, optstr)) != -1) {
@@ -1143,10 +1077,12 @@ int main(int argc, char **argv)
 			break;
 		case 'f':
 			force_mode = 1;
-			if(sscanf(optarg,"%f,%d,%d,%d,%d,%d,%d,%d,%d",
-				&force_clock,&force_hdisplay, &force_hsync_start,&force_hsync_end,&force_htotal,
-				&force_vdisplay, &force_vsync_start, &force_vsync_end, &force_vtotal)!= 9)
+			if(sscanf(optarg,"%f,%hu,%hu,%hu,%hu,%hu,%hu,%hu,%hu",
+				&force_clock,&force_timing.hdisplay, &force_timing.hsync_start,&force_timing.hsync_end,&force_timing.htotal,
+				&force_timing.vdisplay, &force_timing.vsync_start, &force_timing.vsync_end, &force_timing.vtotal)!= 9)
 				usage(argv[0]);
+			force_timing.clock = force_clock*1000;
+
 			break;
 		case 's':
 			sleep_between_modes = atoi(optarg);
@@ -1181,8 +1117,8 @@ int main(int argc, char **argv)
 		test_all_modes = 1;
 
 	for (i = 0; i < ARRAY_SIZE(modules); i++) {
-		fd = drmOpen(modules[i], NULL);
-		if (fd < 0)
+		drm_fd = drmOpen(modules[i], NULL);
+		if (drm_fd < 0)
 			printf("failed to load %s driver.\n", modules[i]);
 		else
 			break;
@@ -1194,94 +1130,53 @@ int main(int argc, char **argv)
 		goto out;
 	}
 
-	u = udev_new();
-	if (!u) {
-		fprintf(stderr, "failed to create udev object\n");
-		ret = -1;
-		goto out_close;
-	}
-
-	uevent_monitor = udev_monitor_new_from_netlink(u, "udev");
-	if (!uevent_monitor) {
-		fprintf(stderr, "failed to create udev event monitor\n");
-		ret = -1;
-		goto out_udev_unref;
-	}
-
-	ret = udev_monitor_filter_add_match_subsystem_devtype(uevent_monitor,
-							      "drm",
-							      "drm_minor");
-	if (ret < 0) {
-		fprintf(stderr, "failed to filter for drm events\n");
-		goto out_udev_mon_unref;
-	}
-
-	ret = udev_monitor_enable_receiving(uevent_monitor);
-	if (ret < 0) {
-		fprintf(stderr, "failed to enable udev event reception\n");
-		goto out_udev_mon_unref;
-	}
-
 	mainloop = g_main_loop_new(NULL, FALSE);
 	if (!mainloop) {
 		fprintf(stderr, "failed to create glib mainloop\n");
 		ret = -1;
-		goto out_mainloop_unref;
+		goto out_close;
 	}
 
-	udevchannel =
-		g_io_channel_unix_new(udev_monitor_get_fd(uevent_monitor));
-	if (!udevchannel) {
-		fprintf(stderr, "failed to create udev GIO channel\n");
-		goto out_mainloop_unref;
-	}
-
-	ret = g_io_add_watch(udevchannel, G_IO_IN | G_IO_ERR, hotplug_event,
-			     u);
-	if (ret < 0) {
-		fprintf(stderr, "failed to add watch on udev GIO channel\n");
-		goto out_udev_off;
+	if (!testdisplay_setup_hotplug()) {
+		fprintf(stderr, "failed to initialize hotplug support\n");
+		goto out_mainloop;
 	}
 
 	stdinchannel = g_io_channel_unix_new(0);
 	if (!stdinchannel) {
 		fprintf(stderr, "failed to create stdin GIO channel\n");
-		goto out_udev_off;
+		goto out_hotplug;
 	}
 
 	ret = g_io_add_watch(stdinchannel, G_IO_IN | G_IO_ERR, input_event,
 			     NULL);
 	if (ret < 0) {
 		fprintf(stderr, "failed to add watch on stdin GIO channel\n");
-		goto out_stdio_off;
+		goto out_stdio;
 	}
 
 	ret = 0;
 
 	if (!update_display()) {
 		ret = 1;
-		goto out_stdio_off;
+		goto out_stdio;
 	}
 
 	if (dump_info || test_all_modes)
-		goto out_stdio_off;
+		goto out_stdio;
 
 	g_main_loop_run(mainloop);
 
-out_stdio_off:
+out_stdio:
 	g_io_channel_shutdown(stdinchannel, TRUE, NULL);
-out_udev_off:
-	g_io_channel_shutdown(udevchannel, TRUE, NULL);
-out_mainloop_unref:
+out_hotplug:
+	testdisplay_cleanup_hotplug();
+out_mainloop:
 	g_main_loop_unref(mainloop);
-out_udev_mon_unref:
-	udev_monitor_unref(uevent_monitor);
-out_udev_unref:
-	udev_unref(u);
 out_close:
 	if (test_plane)
-		disable_planes(fd);
-	drmClose(fd);
+		disable_planes(drm_fd);
+	drmClose(drm_fd);
 out:
 	return ret;
 }
